@@ -31,6 +31,8 @@
  *
  *)
 
+open Mm_base
+
 (** Operations on audio data. *)
 
 (** At given sample rate, number of samples in given amount of time. *)
@@ -61,58 +63,54 @@ module Note : sig
   val a4 : int
   val c5 : int
   val c0 : int
-
   val create : int -> int -> t
-
   val freq : t -> float
-
   val of_freq : float -> t
-
   val name : t -> int
-
   val octave : t -> int
 
   (** Returns note number and octave. *)
   val modulo : t -> int * int
 
   val to_string : t -> string
-
   val of_string : string -> t
 end
 
 (** Operations on mono buffers (with only one channel). *)
 module Mono : sig
   (** A mono buffer. *)
-  type buffer = float array
+  type t = (float, Bigarray.float32_elt, Bigarray.c_layout) Bigarray.Array1.t
 
-  val create : int -> buffer
+  type buffer = t
 
-  val blit : buffer -> int -> buffer -> int -> int -> unit
+  val create : int -> t
+  val make : int -> float -> t
+  val of_array : float array -> t
+  val to_array : t -> float array
+  val sub : t -> int -> int -> t
+  val blit : t -> t -> unit
+  val copy : t -> t
 
-  val copy : buffer -> buffer
+  (** Length in samples. *)
+  val length : t -> int
 
-  val duration : buffer -> int
-
-  val append : buffer -> buffer -> buffer
+  val append : t -> t -> t
 
   (** Clear a portion of a buffer (fill it with zeroes). *)
-  val clear : buffer -> int -> int -> unit
+  val clear : t -> unit
 
-  val amplify : float -> buffer -> int -> int -> unit
+  val amplify : float -> t -> unit
+  val resample : ?mode:[ `Nearest | `Linear ] -> float -> t -> t
+  val clip : t -> unit
+  val noise : t -> unit
 
-  val resample : ?mode:[`Nearest | `Linear] -> float -> buffer -> int -> int -> buffer
+  (** Samplewise add two buffers of the same length. *)
+  val add : t -> t -> unit
 
-  val clip : buffer -> int -> int -> unit
+  val mult : t -> t -> unit
 
-  (** [add b1 o1 b2 o2 len] adds [len] samples of contents of [b2] starting at
-      [o2] to [b1] starting at [o1]. *)
-  val add : buffer -> int -> buffer -> int -> int -> unit
-
-  val mult : buffer -> int -> buffer -> int -> int -> unit
-
-  module Ringbuffer_ext : Ringbuffer.R with type elt = float
-
-  module Ringbuffer : Ringbuffer.R with type elt = float
+  module Ringbuffer_ext : Ringbuffer.R with type buffer = t
+  module Ringbuffer : Ringbuffer.R with type buffer = t
 
   (** Buffers of variable size. These are particularly useful for temporary
       buffers. *)
@@ -120,16 +118,14 @@ module Mono : sig
     type t
 
     val create : int -> t
-
-    val duration : t -> int
-
+    val length : t -> int
     val prepare : t -> int -> buffer
   end
 
   (** Functions for analyzing audio data. *)
   module Analyze : sig
     (** Compute the RMS power of a portion of a buffer. *)
-    val rms : buffer -> int -> int -> float
+    val rms : t -> float
 
     (** Simple implementation of the FFT algorithm. For fastest implementations
 	optimized libraries such as fftw are recommended. *)
@@ -140,13 +136,12 @@ module Mono : sig
       (** Initialize FFT for an analysis of [2^n] samples. *)
       val init : int -> t
 
-      (** Duration of the FFT buffer analysis in samples. *)
-      val duration : t -> int
+      (** Length of the FFT buffer analysis in samples. *)
+      val length : t -> int
 
-      (** [complex_create buf ofs len] create a array of complex numbers of size
-	  [len] by copying data from [buf] from ofset [ofs] (the imaginary part
-	  is null). *)
-      val complex_create : buffer -> int -> int -> Complex.t array
+      (** [complex_create buf] create a array of complex numbers by copying data
+         from [buf] (the imaginary part is null). *)
+      val complex_create : buffer -> Complex.t array
 
       (** Perform an FFT analysis. *)
       val fft : t -> Complex.t array -> unit
@@ -158,49 +153,59 @@ module Mono : sig
 	  to improve the quality of the FFT, see
 	  http://en.wikipedia.org/wiki/Windowing_functions. *)
       module Window : sig
-	val cosine : Complex.t array -> unit
-
-	val hann : Complex.t array -> unit
-
-	val hamming : Complex.t array -> unit
-
-	val lanczos : Complex.t array -> unit
-
-	val triangular : Complex.t array -> unit
-
-	val bartlett_hann : Complex.t array -> unit
-
-	val blackman : ?alpha:float -> t -> Complex.t array -> unit
-
-	val nuttall : t -> Complex.t array -> unit
-
-	val blackman_harris : t -> Complex.t array -> unit
-
-	val blackman_nuttall : t -> Complex.t array -> unit
+        val cosine : Complex.t array -> unit
+        val hann : Complex.t array -> unit
+        val hamming : Complex.t array -> unit
+        val lanczos : Complex.t array -> unit
+        val triangular : Complex.t array -> unit
+        val bartlett_hann : Complex.t array -> unit
+        val blackman : ?alpha:float -> Complex.t array -> unit
+        val nuttall : Complex.t array -> unit
+        val blackman_harris : Complex.t array -> unit
+        val blackman_nuttall : Complex.t array -> unit
       end
 
-      val notes : int -> t -> ?window:(Complex.t array -> unit) -> ?note_min:int -> ?note_max:int -> ?volume_min:float -> ?filter_harmonics:bool -> float array -> int -> int -> (Note.t * float) list
+      val notes :
+        int ->
+        t ->
+        ?note_min:int ->
+        ?note_max:int ->
+        ?volume_min:float ->
+        ?filter_harmonics:bool ->
+        buffer ->
+        (Note.t * float) list
 
       val loudest_note : (Note.t * float) list -> (Note.t * float) option
-
     end
   end
 
   module Effect : sig
     (** A compander following the mu-law (see
 	http://en.wikipedia.org/wiki/Mu-law).*)
-    val compand_mu_law : float -> buffer -> int -> int -> unit
+    val compand_mu_law : float -> t -> unit
 
     class type t =
-    object
-      method process : buffer -> int -> int -> unit
-    end
+      object
+        method process : buffer -> unit
+      end
 
     class amplify : float -> t
-
     class clip : float -> t
 
-    class biquad_filter : int -> [ `Band_pass | `High_pass | `Low_pass | `Notch | `All_pass | `Peaking | `Low_shelf | `High_shelf ] -> ?gain:float -> float -> float -> t
+    class biquad_filter :
+      int
+      -> [ `Band_pass
+         | `High_pass
+         | `Low_pass
+         | `Notch
+         | `All_pass
+         | `Peaking
+         | `Low_shelf
+         | `High_shelf ]
+      -> ?gain:float
+      -> float
+      -> float
+      -> t
 
     (** ADSR (Attack/Decay/Sustain/Release) envelopes. *)
     module ADSR : sig
@@ -220,10 +225,8 @@ module Mono : sig
       val init : unit -> state
 
       val release : state -> state
-
       val dead : state -> bool
-
-      val process : t -> state -> buffer -> int -> int -> state
+      val process : t -> state -> buffer -> state
     end
   end
 
@@ -231,24 +234,23 @@ module Mono : sig
   module Generator : sig
     (** A sound generator. *)
     class type t =
-    object
-      method set_volume : float -> unit
+      object
+        method set_volume : float -> unit
+        method set_frequency : float -> unit
 
-      method set_frequency : float -> unit
+        (** Fill a buffer with generated sound. *)
+        method fill : buffer -> unit
 
-      (** Fill a buffer with generated sound. *)
-      method fill : buffer -> int -> int -> unit
+        (** Same as [fill] but adds the sound to the buffer. *)
+        method fill_add : buffer -> unit
 
-      (** Same as [fill] but adds the sound to the buffer. *)
-      method fill_add : buffer -> int -> int -> unit
+        (** Release the generator (used for generator with envelopes). *)
+        method release : unit
 
-      (** Release the generator (used for generator with envelopes). *)
-      method release : unit
-
-      (** Is the generator still producing sound? This should become false soon
+        (** Is the generator still producing sound? This should become false soon
 	  after release has been triggered. *)
-      method dead : bool
-    end
+        method dead : bool
+      end
 
     (** Generate a sine waveform. *)
     class sine : int -> ?volume:float -> ?phase:float -> float -> t
@@ -263,11 +265,8 @@ module Mono : sig
     class triangle : int -> ?volume:float -> ?phase:float -> float -> t
 
     class white_noise : ?volume:float -> int -> t
-
     class chain : t -> Effect.t -> t
-
     class add : t -> t -> t
-
     class mult : t -> t -> t
 
     (** Apply an ADSR envlope on a generator. *)
@@ -276,95 +275,96 @@ module Mono : sig
 end
 
 (** An audio buffer. *)
-type buffer = Mono.buffer array
+type t = Mono.t array
+
+type buffer = t
 
 (** [create chans len] creates a buffer with [chans] channels and [len] samples
     as duration. *)
-val create : int -> int -> buffer
+val create : int -> int -> t
+
+val make : int -> int -> float -> t
+val of_array : float array array -> t
+val to_array : t -> float array array
 
 (** Create a buffer with the same number of channels and duration as the given
     buffer. *)
-val create_same : buffer -> buffer
+val create_same : t -> t
 
 (** Clear the buffer (sets all the samples to zero). *)
-val clear : buffer -> int -> int -> unit
+val clear : t -> unit
 
 (** Copy the given buffer. *)
-val copy : buffer -> buffer
+val copy : t -> t
 
-val append : buffer -> buffer -> buffer
+val append : t -> t -> t
+val channels : t -> int
 
-val channels : buffer -> int
-
-(** Duration of a buffer in samples. *)
-val duration : buffer -> int
+(** Length of a buffer in samples. *)
+val length : t -> int
 
 (** Convert a buffer to a mono buffer by computing the mean of all channels. *)
-val to_mono : buffer -> Mono.buffer
+val to_mono : t -> Mono.t
 
 (** Convert a mono buffer into a buffer. Notice that the original mono buffer is
     not copied an might thus be modified afterwards. *)
-val of_mono : Mono.buffer -> buffer
+val of_mono : Mono.t -> t
+
+val interleave :
+  t -> (float, Bigarray.float32_elt, Bigarray.c_layout) Bigarray.Array1.t
+
+val deinterleave :
+  int -> (float, Bigarray.float32_elt, Bigarray.c_layout) Bigarray.Array1.t -> t
 
 module U8 : sig
-  val of_audio : buffer -> int -> string -> int -> int -> unit
-
-  val to_audio : string -> int -> buffer -> int -> int -> unit
+  val size : int -> int -> int
+  val of_audio : t -> Bytes.t -> int -> unit
+  val to_audio : string -> int -> t -> unit
 end
 
 module S16LE : sig
+  val size : int -> int -> int
   val length : int -> int -> int
-
-  val duration : int -> int -> int
-
-  val of_audio : buffer -> int -> string -> int -> int -> unit
-
-  val make : buffer -> int -> int -> string
-
-  val to_audio : string -> int -> buffer -> int -> int -> unit
+  val of_audio : t -> Bytes.t -> int -> unit
+  val make : t -> string
+  val to_audio : string -> int -> t -> unit
 end
 
 module S16BE : sig
+  val size : int -> int -> int
   val length : int -> int -> int
-
-  val duration : int -> int -> int
-
-  val of_audio : buffer -> int -> string -> int -> int -> unit
-
-  val make : buffer -> int -> int -> string
-
-  val to_audio : string -> int -> buffer -> int -> int -> unit
+  val of_audio : t -> Bytes.t -> int -> unit
+  val make : t -> string
+  val to_audio : string -> int -> t -> unit
 end
 
 module S24LE : sig
-  val of_audio : buffer -> int -> string -> int -> int -> unit
-
-  val to_audio : string -> int -> buffer -> int -> int -> unit
+  val size : int -> int -> int
+  val of_audio : t -> Bytes.t -> int -> unit
+  val to_audio : string -> int -> t -> unit
 end
 
 module S32LE : sig
-  val of_audio : buffer -> int -> string -> int -> int -> unit
-
-  val to_audio : string -> int -> buffer -> int -> int -> unit
+  val size : int -> int -> int
+  val of_audio : t -> Bytes.t -> int -> unit
+  val to_audio : string -> int -> t -> unit
 end
 
-val resample : ?mode:[`Nearest | `Linear] -> float -> buffer -> int -> int -> buffer
-
-(** Same as [Array.blit] for audio data. *)
-val blit : buffer -> int -> buffer -> int -> int -> unit
-
-val clip : buffer -> int -> int -> unit
+val resample : ?mode:[ `Nearest | `Linear ] -> float -> t -> t
+val blit : t -> t -> unit
+val sub : t -> int -> int -> t
+val clip : t -> unit
+val noise : t -> unit
 
 (** Amplify a portion of the buffer by a given coefficient. *)
-val amplify : float -> buffer -> int -> int -> unit
+val amplify : float -> t -> unit
 
 (** Pan a stereo buffer from left to right (the buffer should have exactly two
     channels!). The coefficient should be between [-1.] and [1.]. *)
-val pan : float -> buffer -> int -> int -> unit
+val pan : float -> t -> unit
 
-val add : buffer -> int -> buffer -> int -> int -> unit
-
-val add_coeff : buffer -> int -> float -> buffer -> int -> int -> unit
+val add : t -> t -> unit
+val add_coeff : t -> float -> t -> unit
 
 (** Buffers of variable size. These are particularly useful for temporary
     buffers. *)
@@ -375,8 +375,8 @@ module Buffer_ext : sig
       samples. *)
   val create : int -> int -> t
 
-  (** Current duration (in samples) of the buffer. *)
-  val duration : t -> int
+  (** Current length (in samples) of the buffer. *)
+  val length : t -> int
 
   (** Make sure that the buffer can hold at least a given number of samples. *)
   val prepare : t -> ?channels:int -> int -> buffer
@@ -391,122 +391,134 @@ module Ringbuffer : sig
   val create : int -> int -> t
 
   val channels : t -> int
-
   val read_space : t -> int
-
   val write_space : t -> int
-
   val read_advance : t -> int -> unit
-
   val write_advance : t -> int -> unit
-
-  val peek : t -> buffer -> int -> int -> unit
-
-  val read : t -> buffer -> int -> int -> unit
-
-  val write : t -> buffer -> int -> int -> unit
-
-  val transmit : t -> (buffer -> int -> int -> int) -> int
+  val peek : t -> buffer -> unit
+  val read : t -> buffer -> unit
+  val write : t -> buffer -> unit
+  val transmit : t -> (buffer -> int) -> int
 end
 
 module Ringbuffer_ext : sig
   type t
 
   val create : int -> int -> t
-
   val channels : t -> int
-
   val read_space : t -> int
-
   val write_space : t -> int
-
   val read_advance : t -> int -> unit
-
   val write_advance : t -> int -> unit
-
-  val peek : t -> buffer -> int -> int -> unit
-
-  val read : t -> buffer -> int -> int -> unit
-
-  val write : t -> buffer -> int -> int -> unit
-
-  val transmit : t -> (buffer -> int -> int -> int) -> int
+  val peek : t -> buffer -> unit
+  val read : t -> buffer -> unit
+  val write : t -> buffer -> unit
+  val transmit : t -> (buffer -> int) -> int
 end
 
 module Analyze : sig
-  val rms : buffer -> int -> int -> float array
+  val rms : t -> float array
 end
 
 (** Audio effects. *)
 module Effect : sig
   (** A possibly stateful audio effect. *)
   class type t =
-  object
-    (** Apply the effect on a buffer. *)
-    method process : buffer -> int -> int -> unit
-  end
+    object
+      (** Apply the effect on a buffer. *)
+      method process : buffer -> unit
+    end
 
   class chain : t -> t -> t
-
   class of_mono : int -> (unit -> Mono.Effect.t) -> t
 
   class type delay_t =
-  object
-    inherit t
-    method set_delay : float -> unit
-    method set_feedback : float -> unit
-  end
+    object
+      inherit t
+      method set_delay : float -> unit
+      method set_feedback : float -> unit
+    end
 
   (** [delay chans samplerate d once feedback] creates a delay operator for
       buffer with [chans] channels at [samplerate] samplerate with [d] as delay
       in seconds and [feedback] as feedback. If [once] is set to [true] only one
       echo will be heard (no feedback). *)
-  val delay : int -> int -> float -> ?once:bool -> ?ping_pong:bool -> float -> delay_t
+  val delay :
+    int -> int -> float -> ?once:bool -> ?ping_pong:bool -> float -> delay_t
 
   (** Hardknee compressor with RMS look-ahead envelope calculation and
       adjustable attack/decay. Given parameters are [attack] and [release] in
       seconds, [ratio] n means n:1 compression, [threshold] and [knee] in dB,
       and [rms_window] in second is the duration for RMS acquisition. [gain] is
       an additional pre-gain. *)
-  class compress : ?attack:float -> ?release:float -> ?threshold:float -> ?ratio:float -> ?knee:float -> ?rms_window:float -> ?gain:float -> int -> int ->
-  object
-    inherit t
-    method set_attack : float -> unit
-    method set_gain : float -> unit
-    method set_knee : float -> unit
-    method set_ratio : float -> unit
-    method set_release : float -> unit
-    method set_threshold : float -> unit
-    method reset : unit
-  end
+  class compress :
+    ?attack:float
+    -> ?release:float
+    -> ?threshold:float
+    -> ?ratio:float
+    -> ?knee:float
+    -> ?rms_window:float
+    -> ?gain:float
+    -> int
+    -> int
+    -> object
+         inherit t
+         method set_attack : float -> unit
+         method set_gain : float -> unit
+         method set_knee : float -> unit
+         method set_ratio : float -> unit
+         method set_release : float -> unit
+         method set_threshold : float -> unit
+         method reset : unit
+       end
 
   (** A biquadratic filter. [gain] in dB is only used by peaking, low and high shelves. *)
-  class biquad_filter : int -> int -> [ `Band_pass | `High_pass | `Low_pass | `Notch | `All_pass | `Peaking | `Low_shelf | `High_shelf ] -> ?gain:float -> float -> float -> t
+  class biquad_filter :
+    int
+    -> int
+    -> [ `Band_pass
+       | `High_pass
+       | `Low_pass
+       | `Notch
+       | `All_pass
+       | `Peaking
+       | `Low_shelf
+       | `High_shelf ]
+    -> ?gain:float
+    -> float
+    -> float
+    -> t
 
-  val auto_gain_control : int -> int -> ?rms_target:float -> ?rms_window:float -> ?kup:float -> ?kdown:float -> ?rms_threshold:float -> ?volume_init:float -> ?volume_min:float -> ?volume_max:float -> unit -> t
+  val auto_gain_control :
+    int ->
+    int ->
+    ?rms_target:float ->
+    ?rms_window:float ->
+    ?kup:float ->
+    ?kdown:float ->
+    ?rms_threshold:float ->
+    ?volume_init:float ->
+    ?volume_min:float ->
+    ?volume_max:float ->
+    unit ->
+    t
 end
 
 (** Sound generators. *)
 module Generator : sig
-  val white_noise : buffer -> int -> int -> unit
+  val white_noise : t -> unit
 
   class type t =
-  object
-    method set_volume : float -> unit
-
-    method set_frequency : float -> unit
-
-    method fill : buffer -> int -> int -> unit
-
-    method fill_add : buffer -> int -> int -> unit
-
-    method release : unit
-
-    method dead : bool
-  end
+    object
+      method set_volume : float -> unit
+      method set_frequency : float -> unit
+      method fill : buffer -> unit
+      method fill_add : buffer -> unit
+      method release : unit
+      method dead : bool
+    end
 
   class of_mono : Mono.Generator.t -> t
-
   class chain : t -> Effect.t -> t
 end
 
@@ -524,41 +536,40 @@ module IO : sig
 
   module Reader : sig
     class type t =
-    object
-    (** Number of channels. *)
-      method channels : int
+      object
+        (** Number of channels. *)
+        method channels : int
 
-    (** Sample rate in samples per second. *)
-      method sample_rate : int
+        (** Sample rate in samples per second. *)
+        method sample_rate : int
 
-    (** Duration in samples. *)
-      method duration : int
+        (** Length in samples. *)
+        method length : int
 
-    (** Duration in seconds. *)
-      method duration_time : float
+        (** Duration in seconds. *)
+        method duration : float
 
-    (** Seek to a given sample. *)
-      method seek : int -> unit
+        (** Seek to a given sample. *)
+        method seek : int -> unit
 
-    (** Close the file. This method should only be called once. The members of
+        (** Close the file. This method should only be called once. The members of
 	the object should not be accessed anymore after this method has been
 	called. *)
-      method close : unit
+        method close : unit
 
-      method read : buffer -> int -> int -> int
-    end
+        method read : buffer -> int
+      end
 
-  (** Create a reader object from a wav file. *)
+    (** Create a reader object from a wav file. *)
     class of_wav_file : string -> t
   end
 
   module Writer : sig
     class type t =
-    object
-      method write : buffer -> int -> int -> unit
-
-      method close : unit
-    end
+      object
+        method write : buffer -> unit
+        method close : unit
+      end
 
     (** Create a writer to a file in WAV format with given number of channels,
         sample rate and file name.*)
@@ -567,23 +578,23 @@ module IO : sig
 
   module RW : sig
     class type t =
-    object
-      method read : buffer -> int -> int -> unit
+      object
+        method read : buffer -> unit
+        method write : buffer -> unit
+        method close : unit
+      end
 
-      method write : buffer -> int -> int -> unit
-
-      method close : unit
-    end
-
-    class virtual bufferized : int -> min_duration:int -> fill_duration:int -> max_duration:int -> drop_duration:int ->
-    object
-      method virtual io_read : buffer -> int -> int -> unit
-
-      method virtual io_write : buffer -> int -> int -> unit
-
-      method read : buffer -> int -> int -> unit
-
-      method write : buffer -> int -> int -> unit
-    end
+    class virtual bufferized :
+      int
+      -> min_duration:int
+      -> fill_duration:int
+      -> max_duration:int
+      -> drop_duration:int
+      -> object
+           method virtual io_read : buffer -> unit
+           method virtual io_write : buffer -> unit
+           method read : buffer -> unit
+           method write : buffer -> unit
+         end
   end
 end
